#include "llvm/Pass.h"
#include "llvm/IR/Function.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/IR/LegacyPassManager.h"
#include "llvm/Transforms/IPO/PassManagerBuilder.h"
#include "llvm/Analysis/Trace.h"
#include "llvm/IR/BasicBlock.h"
#include "llvm/IR/Instructions.h"

using namespace llvm;

namespace
{
struct SkeletonPass : public FunctionPass
{
  static char ID;
  SkeletonPass() : FunctionPass(ID) {}

  virtual bool runOnFunction(Function &F)
  {
    errs() << "Function " << F.getName() << "!\n";

    for (Function::BasicBlockListType::iterator b = F.begin(), be = F.end(); b != be; ++b)
    {
      for (llvm::BasicBlock::InstListType::iterator j = b->begin(), je = b->end(); j != je; ++j)
      {
        Instruction *ii = &*j;
        //        errs() << *ii << "\n";
        errs() << "opcode: " << *ii << ", in function: " << (*ii).getFunction()->getName() << "\n";
        if (CallInst *c = dyn_cast<CallInst>(ii))
        {
          errs() << "call to:" << c->getCalledFunction()->getName() << "\n";
        }
      }
    }
    return false;
  }
};
} // namespace

char SkeletonPass::ID = 0;

static RegisterPass<SkeletonPass> X("callgirl", "A politically incorrect LLVM pass",
                                    true /* Only looks at CFG */,
                                    true /* Analysis Pass */);

// Automatically enable the pass.
// http://adriansampson.net/blog/clangpass.html
static void registerSkeletonPass(const PassManagerBuilder &,
                                 legacy::PassManagerBase &PM)
{
  PM.add(new SkeletonPass());
}
static RegisterStandardPasses
    RegisterMyPass(PassManagerBuilder::EP_EarlyAsPossible,
                   registerSkeletonPass);
